/* REXX */                                                                      
/*-------------------------------------------------------------------*/         
/*                                                                              
        NAME: CATDB2CP                                                          
 Description: This rexx exec is used to find any conflict between OAM           
              DB2 collection table and OAM catalog collection entries.          
                                                                                
       Input: - Catalog name that contains OAM collection entries.              
              - DB2 SSID                                                        
              - Wildcard for collection entry names (Optional)    @04A          
              - Output data set name.                                           
                                                                                
                                                                                
      Output: - A data set containing those OAM collection entries @04C         
                that exist in catalog but not in DB2 collection table.          
              - A data set containing those OAM collection entries              
                that have same collection name, but different                   
                collection IDs between DB2 table and catalog.      @04A         
                                                                                
 Return Codes:                                                                  
       4 - no conflict found or catalog data set is empty                       
       8 - error occured                                                        
                                                                                
  Written by: Junfeng Xu 08/22/2017                                             
                                                                                
  CHANGE ACTIVITY:                                                              
  $01 210 12102017 TUCAED: Added new case found by customer:                    
                           Same collection name in catalog and DB2,             
                           but different collection IDs           @01A          
  $02 210 07172018 TUCJFX: Update CATSRCHI invocation to reduce system          
                           storage usage & add colnwc input option @02A         
  $03 210 07262018 TUCJFX: Add checking for DB2 connecting environment          
                                                                   @03A         
  $04 210 07272018 TUCJFX: Remove existing dataset deleting for output          
                                                                   @04A         
  $05 210 09252018 TUCJFX: Support input parms on program invocation            
                                                                   @05A         
*/                                                                              
/*-------------------------------------------------------------------*/         
                                                                                
 arg parmin                          /* Save any parms passed to exec*/         
 upper parmin                        /* Convert parms to uppercase   */         
 call Initialize                     /* Initialize exec variables    */         
 if retcode = 0 then                                                            
   call parse_ds                     /* Call parse data sets         */         
 if retcode <= 4 then                                                           
   call catdb2_compare               /* compare cat entries with db2 */         
                                                               /* @01D*/        
 exit                                                                           
                                                                                
/**************************** Subroutines ****************************/         
/*-------------------------------------------------------------------*/         
/* Initialize the variables used by this exec                        */         
/*-------------------------------------------------------------------*/         
Initialize:                                                                     
 debugging = 0                       /* set this field one to debug  */         
 retcode = 0                                                                    
 dsname    = ''                      /* Save dsn returned by IGGCSI00*/         
 exitrc    = 0                       /* Set exit rc to zero          */         
 batch_exec = 0                      /* on if inputs on cmd line @05A*/         
                                                                                
 say 'CATDB2CP: OAM catalog entries and DB2 collection table ' ||,              
 'entries comparison tool. Dated: 09/25/2018'                  /* @05C*/        
 say '---------------------------------------------------------------'          
 say 'If the case is found where there are collection' ||,     /* @01C*/        
     ' entries in catalog, but not in DB2...'                  /* @01A*/        
 say 'Then an output data set will be created that contains:'  /* @01A*/        
 say 'a) all collection entries in the catalog that are missing' ||,            
 ' from the OAM DB2 collection table'                                           
 say 'b) collection entries in the catalog whose collection' ||,                
 ' ID is associated with a different collection name in the' ||,                
 ' OAM DB2 collection table'                                                    
 say '---------------------------------------------------------------'          
 say ' '                                                                        
 say 'If the case is found where there is a collection' ||,    /* @01A*/        
     ' name in catalog and the same collection name in' ||,    /* @01A*/        
     ' DB2, but... the collection IDs are DIFFERENT...'        /* @01A*/        
 say 'Then an output data set will be created that contains:'  /* @01A*/        
 say 'a) all the collection entries in the catalog and DB2' ||,/* @01A*/        
     ' that have the same name, but different IDs.'            /* @01A*/        
 say 'b) collection entries in the catalog whose collection'||,/* @01A*/        
 ' ID is associated with a different collection name in the'||,/* @01A*/        
 ' OAM DB2 collection table'                                   /* @01A*/        
 say '--------------------------------' ||,                    /* @01A*/        
     '--------------------------------'                        /* @01A*/        
 say ' '                                                       /* @01A*/        
                                                                                
 if words(parmin) > 0 then           /* Were parms passed to exec?   */         
  do                                 /* Yes,                         */         
   if words(parmin) < 5 then         /* 5 required input parms    @05A*/        
    do                                                         /* @05A*/        
     say 'Error: some required input parameters are missing. ' ||,              
         'Program returned.'                                   /* @05A*/        
     retcode = 8                                               /* @05A*/        
     return                                                    /* @05A*/        
    end                                                        /* @05A*/        
   batch_exec = 1                                              /* @05A*/        
   parse var parmin catn db2ssid colnwc outputds outputds2 extra/*@05C*/        
   say 'User input catalog name:' catn                                          
   say 'User input DB2 SSID:' db2ssid                                           
   say 'User input collection name wildcard:' colnwc           /* @05A*/        
   if colnwc = 'NA' | colnwc = 'N/A' then                      /* @05A*/        
    do                                                         /* @05A*/        
     say "Warning: if one has a big catalog and don't have a "||,               
         "collection name wildcard, this tool may abend because "||,            
         "of running out of storage space to contain the catalog. "             
                                                               /* @05A*/        
     colnwc = '**'                                             /* @05A*/        
    end                                                        /* @05A*/        
   if SYSDSN("'"outputds"'") = 'OK' then                       /* @05A*/        
    do                                                         /* @05A*/        
     say 'Error: Inputted data set name' outputds 'exists. ' ||,                
         'Program returned.'                                   /* @05A*/        
     retcode = 8                                               /* @05A*/        
     return                                                    /* @05A*/        
    end                                                        /* @05A*/        
   else                                                        /* @05A*/        
     say 'Output data set name for missing collection entries:' outputds        
   if SYSDSN("'"outputds2"'") = 'OK' then                      /* @05A*/        
    do                                                         /* @05A*/        
     say 'Error: Inputted data set name' outputds2 'exists. ' ||,               
         'Program returned.'                                   /* @05A*/        
     retcode = 8                                               /* @05A*/        
     return                                                    /* @05A*/        
    end                                                        /* @05A*/        
   else                                                        /* @05A*/        
     say 'Output data set name for same collection name different ' ||,         
         'collection ID entries:' outputds2                    /* @05A*/        
   say ' '                                                                      
   say '------------------------------------------------------------'           
   if words(extra) > 0 then                                                     
    do                                                                          
     say 'Error: Too many parameters. ('extra')'                                
     retcode = 8                                                                
     return                                                                     
    end                                                                         
  end                                                                           
 else                                                                           
  if SYSVAR(SYSENV) = 'FORE' then    /* if running in foreground     */         
   do                                                                           
    catn = ''                                                 /* @02A*/         
    do while catn = ''                                        /* @02A*/         
     say 'Enter input catalog name:'                                            
     pull catn                       /* get catalog ds name          */         
     if catn = '' then                                        /* @02A*/         
      say 'Input field cannot be empty.'                      /* @02A*/         
    end                                                       /* @02A*/         
    say 'User input catalog name:' catn                                         
                                                                                
    db2ssid = ''                                              /* @02A*/         
    do while db2ssid = ''                                     /* @02A*/         
     say 'Enter the DB2 SSID:'                                                  
     pull db2ssid                    /* get db2ssid                  */         
     if db2ssid = '' then                                     /* @02A*/         
      say 'Input field cannot be empty.'                      /* @02A*/         
    end                                                       /* @02A*/         
    say 'User input DB2 SSID:' db2ssid                                          
                                                                                
    colnwc = ''                                               /* @02A*/         
    do while colnwc = ''                                      /* @02A*/         
     say "Enter collection name wildcard" /* get coln wildcard   @02A*/         
     say "Valid wildcard examples: "                          /* @02A*/         
     say "    OAM.**"                                         /* @02A*/         
     say "    OAM.collection*.**"                             /* @02A*/         
     say "    collection*"                                    /* @02A*/         
     say "(Put 'NA' if don't have a wildcard):"               /* @02A*/         
     pull colnwc                                              /* @02A*/         
     if colnwc = ''then                                       /* @02A*/         
      say 'Input field cannot be empty.'                      /* @02A*/         
     else if colnwc = 'NA' | colnwc = 'N/A' then              /* @02A*/         
      do                                                      /* @02A*/         
       say "Warning: if one has a big catalog and don't have a "||,             
           "collection name wildcard, this tool may abend because "||,          
           "of running out of storage space to contain the catalog. "           
                                                              /* @02A*/         
       colnwc = '**'                                          /* @02A*/         
      end                                                     /* @02A*/         
     else                                                     /* @02A*/         
      say 'User input collection name wildcard:' colnwc       /* @02A*/         
    end                                                       /* @02A*/         
   end                                                                          
 else                                                                           
  do                                                                            
   say 'Error: Program running in invalid environment.'       /* @05C*/         
   retcode = 8                                                                  
   return                                                                       
  end                                                                           
                                                                                
return                                                                          
                                                                                
/*------------------------------------------------------------------*/          
/* Parsing the two inputs                                           */          
/*------------------------------------------------------------------*/          
Parse_ds:                                                                       
                                                                                
myfile. = ''                                                                    
/* query specified catalog into myfile.                              */         
if catn <> '' then                                                              
do                                                                              
 ADDRESS TSO                                                                    
 x = outtrap('myfile.')                                                         
 "CATSRCHI "colnwc" CAT("catn") WORK AREA(900000) TYPES(A)    /* @02C*/         
   FIELDNAMES(NVSMATTR STORCLAS MGMTCLAS VOLOAMDN)"           /* @01A*/         
 x = outtrap('off')                                                             
end                                                                             
/* parsing catalog OAM entries input                                 */         
                                                                                
if myfile.0 = 0 then                                                            
 do                                                                             
  say 'Error: Catalog entries output is empty'                                  
  retcode = 8                                                                   
  return                                                                        
 end                                                                            
                                                                                
if debugging = 2 then                                         /* @02A*/         
 do                                                                             
  say 'catsrchi output:'                                                        
  i = 1                                                                         
  do while i < myfile.0                                                         
   say myfile.i                                                                 
   i = i + 1                                                                    
  end                                                                           
 end                                                                            
                                                                                
 /* some lines in the data set have weird "1" at the beginning       */         
 /* So folloing call is needed to get rid of the "1"                 */         
call Clean1                                                                     
                                                                                
i = 1                                                                           
cat_entry_num = 0                                                               
do while i <= myfile.0         /* myfile.0 is the total line number  */         
 parse var myfile.i key1 key2 key3 extra                                        
 if key1 = 'NVSMATTR' & key3 = "c'O'" then /* find an OAM entry      */         
  do                                                                            
   cat_entry_num = cat_entry_num + 1                                            
   j = i - 1                               /* grabbing name         */          
   parse var myfile.j key1 key2 extra                                           
   oamcat.cat_entry_num.name = key2                                             
   j = i + 1                               /* grabbing sc           */          
   parse var myfile.j key1 key2 key3 extra                                      
   sclen = x2d(substr(key3,3,4)) * 2                                            
   oamcat.cat_entry_num.sc = x2c(substr(key3,7,sclen))                          
   j = i + 2                               /* grabbing mc           */          
   parse var myfile.j key1 key2 key3 extra                                      
   mclen = x2d(substr(key3,3,4)) * 2                                            
   oamcat.cat_entry_num.mc = x2c(substr(key3,7,mclen))                          
   j = i + 3                               /* grabbing grpn & ID    */          
   parse var myfile.j key1 key2 key3                                            
   oamcat.cat_entry_num.id = x2d(substr(key3,7,4))                              
   groupln = x2d(substr(key3,11,4)) * 2                                         
   oamcat.cat_entry_num.group = x2c(substr(key3,15,groupln))                    
  end                                                                           
 i = i + 1                                                                      
end                                                                             
                                                                                
if cat_entry_num = 0 then                                                       
 do                                                                             
  say 'Error: no OAM collection entry in this catalog' ||,    /* @02C*/         
      ' or inputted collection entry wildcard is not valid.'  /* @02A*/         
  retcode = 8                                                                   
  return                                                                        
 end                                                                            
                                                                                
if debugging = 1 then                                                           
do                                                                              
 say 'OAM collection entries in catalog:'                                       
 i = 1                                                                          
 do while i <= cat_entry_num                                                    
  say oamcat.i.id oamcat.i.name oamcat.i.sc oamcat.i.mc oamcat.i.group          
  i = i + 1                                                                     
 end                                                                            
end                                                                             
                                                                                
/* parsing OAM DB2 collection table                                  */         
 sqlcode = 0                                                 /* @03A*/          
 RC = 0                                                      /* @03A*/          
 S_RC = 0                                                    /* @03A*/          
 SUBSYS = db2ssid                                                               
 SQLSTMT = 'SELECT * FROM OAMADMIN.CBR_COLLECTION_TBL'                          
 address TSO "SUBCOM DSNREXX"          /* testing the existence of              
                                                   rexx/db2 interface*/         
 say 'RC from SUBCOM DSNREXX is ' RC                         /* @03A*/          
 if RC <> 0 then         /* if not exit, add DSNREXX to the table */            
  do                                                                            
   S_RC = RXSUBCOM('ADD','DSNREXX','DSNREXX')                                   
   say 'RC from RXSUBCOM is ' S_RC                             /* @03A*/        
  end                                                                           
 if S_RC <> 0 then       /* if DSNREXX is failed adding to table  @03A*/        
  do                                                           /* @03A*/        
   retcode = 8                                                 /* @03A*/        
   return                                                      /* @03A*/        
  end                                                          /* @03A*/        
                                                                                
 address DSNREXX "CONNECT" SUBSYS                                               
 say 'RC from DSNREXX CONNECT is ' RC                                           
                                                                                
 if sqlcode = 0 & RC = 0 then                                  /* @03C*/        
  do                                                                            
   address DSNREXX "EXECSQL DECLARE C1 CURSOR FOR S1"                           
   address DSNREXX "EXECSQL PREPARE S1 INTO:OUTPUT FROM:SQLSTMT"                
   address DSNREXX "EXECSQL OPEN C1"                                            
   j = 1                                                                        
   myfile2. = ''                                                                
   do until (sqlcode <> 0) /* getting db2 table into an array */                
    address DSNREXX "EXECSQL FETCH C1 USING DESCRIPTOR:OUTPUT"                  
    if sqlcode = 0 then do                                                      
     line = ''                                                                  
     i = 1                                                                      
     do while i <= output.sqld then                                             
      line = line output.i.sqldata                                              
      i = i + 1                                                                 
     end                                                                        
     myfile2.j = line                                                           
     j = j + 1                                                                  
    end                                                                         
   end                                                                          
   address DSNREXX "EXECSQL CLOSE C1"                                           
   address DSNREXX "DISCONNECT"                                                 
   S_RC = RXSUBCOM('DELETE','DSNREXX','DSNREXX')             /* @03A*/          
  end                                                                           
 else                                                                           
  do                                                                            
   say 'Error: DB2 connection error.'                        /* @03C*/          
   if sqlcode <> 0 then                                      /* @03A*/          
    say 'sqlcode: ' sqlcode                                  /* @03A*/          
   if RC <> 0 then                                           /* @03A*/          
    say 'RC: ' RC                                            /* @03A*/          
   retcode = 8                                                                  
   return                                                                       
  end                                                                           
                                                                                
myfile2_count = 1       /* counting how many entries in this db2 */             
do while myfile2.myfile2_count <> ''                                            
 myfile2_count = myfile2_count + 1                                              
end                                                                             
myfile2.0 = myfile2_count - 1                                                   
                                                                                
if myfile2.0 = 0 then                                                           
 do                                                                             
  say 'Warning: no OAM collection in entered DB2 collection table'              
  retcode = 4                                                                   
 end                                                                            
                                                                                
db2_entry_num = 1                                                               
do while db2_entry_num <= myfile2.0 /* myfile2.0: total line number */          
 parse var myfile2.db2_entry_num key1 key2 key3 key4 key5 extra                 
 oamdb2.db2_entry_num.sc = key1                                                 
 oamdb2.db2_entry_num.mc = key2                                                 
 oamdb2.db2_entry_num.group = key3                                              
 oamdb2.db2_entry_num.id = key4                                                 
 oamdb2.db2_entry_num.name = key5                                               
 db2_entry_num = db2_entry_num + 1                                              
end                                                                             
db2_entry_num = db2_entry_num - 1                                               
                                                                                
if db2_entry_num <> 0 & debugging = 1 then                                      
do                                                                              
 say 'OAM collection entries in DB2 table:'                                     
 i = 1                                                                          
 do while i <= db2_entry_num                                                    
  say oamdb2.i.id oamdb2.i.name oamdb2.i.sc oamdb2.i.mc oamdb2.i.group          
  i = i + 1                                                                     
 end                                                                            
end                                                                             
                                                                                
return                                                                          
                                                                                
/*-------------------------------------------------------------------*/         
/* Comparing the two arrays here                                     */         
/*-------------------------------------------------------------------*/         
catdb2_compare:                                                                 
 cati = 1                                                                       
 found_in_db2 = 0                                                               
 diff_col_id = 0                                               /* @01A*/        
 conflict_found = 0                                                             
 outputi = 9                   /* actual content starts at line 3    */         
 conflict. = ''                                                                 
                                                                                
 /* header info for the output file. 'conflict' is an array that */             
 /* will write into the output data set.                         */             
 conflict.1 = 'The following collection entries are found in catalog'||,        
              ' but are missing from the OAM DB2 collection table.'             
 conflict.2 = ' '                                                               
 conflict.3 = 'Note: if there is any collection entry ' ||,                     
              'in the OAM DB2 collection table that has the same' ||,           
              ' collection ID as a missing entry, '                             
 conflict.4 = '      it will be output below ' ||,                              
              'inside parentheses. '                                            
 conflict.5 = ' '                                                               
 conflict.6 = left('+',125,'+')                                                 
 conflict.7 = 'COLLECTION ID ',                                                 
              left('COLLECTION NAME',44),                                       
              'STORAGE CLASS ',                                                 
              'MANAGEMENT CLASS ',                                              
              'STORAGE GROUP '                                                  
 conflict.8 = left('-',125,'-')                                                 
                                                                                
 /* Header info for the second output file for same collection    @01A*/        
 /* name - different collection ID case.                          @01A*/        
 conflict2_found = 0                                           /* @01A*/        
 output2i = 11                                                 /* @01A*/        
 conflict2. = ''                                               /* @01A*/        
 conflict2.1= left('-',125,'-')                                /* @01A*/        
 conflict2.2 = 'The following collection entries are' ||,      /* @01A*/        
               ' found in catalog and are also found' ||,      /* @01A*/        
               ' in the DB2 collection table with' ||,         /* @01A*/        
               ' a DIFFERENT collection ID.'                   /* @01A*/        
 conflict2.3 = ' '                                             /* @01A*/        
 conflict2.4 = 'Note: if there is any collection entry ' ||,   /* @01A*/        
              'in the OAM DB2 collection table that has ' ||,  /* @01A*/        
              'the same collection ID as a catalog entry, '    /* @01A*/        
 conflict2.5 = '      it will be output below ' ||,            /* @01A*/        
              'inside parentheses. '                           /* @01A*/        
 conflict2.6 = ' '                                             /* @01A*/        
 conflict2.7 = left('+',125,'+')                               /* @01A*/        
 conflict2.8 = '               COLLECTION ID ',                /* @01A*/        
              left('COLLECTION NAME',44),                      /* @01A*/        
              'STORAGE CLASS ',                                /* @01A*/        
              'MANAGEMENT CLASS ',                             /* @01A*/        
              'STORAGE GROUP '                                 /* @01A*/        
 conflict2.9 = left('-',125,'-')                               /* @01A*/        
 conflict2.10= ' '                                             /* @01A*/        
                                                                                
 /* start from the 1st entry in catalog entry list to compare to    */          
 /* each one in DB2 collection table list                           */          
                                                                                
 do while (cati <= cat_entry_num)                                               
                                                                                
  /* if still in the first half of the catalog list, compare to db2 */          
  /* list from the beginning to the end                             */          
                                                                                
  if cati <= (cat_entry_num/2) then                                             
   do                                                                           
    db2i = 1                                                                    
    do while db2i <= db2_entry_num                                              
     if oamcat.cati.name = oamdb2.db2i.name then                                
      do                                                                        
       found_in_db2 = 1                                                         
       if oamcat.cati.id <> oamdb2.db2i.id then                /* @01A*/        
        do                                                     /* @01A*/        
         diff_col_id = 1                                       /* @01A*/        
        end                                                    /* @01A*/        
       leave                                                                    
      end                                                                       
     else                                                                       
      db2i = db2i + 1                                                           
    end                                                                         
   end                                                                          
                                                                                
  /* if in the second half of the catalog list, compare to db2 list */          
  /* from the end backward to the beginning                         */          
                                                                                
  else                                                                          
   do                                                                           
    db2i = db2_entry_num                                                        
    do while db2i > 0                                                           
     if oamcat.cati.name = oamdb2.db2i.name then                                
      do                                                                        
       found_in_db2 = 1                                                         
       if oamcat.cati.id <> oamdb2.db2i.id then                /* @01A*/        
        do                                                     /* @01A*/        
         diff_col_id = 1                                       /* @01A*/        
        end                                                    /* @01A*/        
       leave                                                                    
      end                                                                       
     else                                                                       
      db2i = db2i - 1                                                           
    end                                                                         
   end                                                                          
                                                                                
  /* if exists in catalog, but not found in DB2, output that entry  */          
                                                                                
  if found_in_db2 = 0 then                                                      
   do                                                                           
    conflict.outputi = center(oamcat.cati.id,14),                               
                       left(oamcat.cati.name,44),                               
                       center(oamcat.cati.sc,14),                               
                       center(oamcat.cati.mc,17),                               
                       center(oamcat.cati.group,14)                             
    outputi = outputi + 1                                                       
    conflict_found = 1                                                          
    db2i = 1                                                                    
    do while db2i <= db2_entry_num                                              
     if oamcat.cati.id = oamdb2.db2i.id then                                    
      do                                                                        
       conflict.outputi = '                  (Collection entry ' ||,            
                          oamdb2.db2i.name                                      
       outputi = outputi + 1                                                    
       conflict.outputi = '                   ' ||,                             
                          'in DB2 with the same collection ID ' ||,             
                          oamdb2.db2i.id ||,                                    
                          ' - SC: ' ||,                                         
                          oamdb2.db2i.sc ||,                                    
                          ' - MC: ' ||,                                         
                          oamdb2.db2i.mc ||,                                    
                          ' - SG: ' ||,                                         
                          oamdb2.db2i.group ||,                                 
                          ') '                                                  
       outputi = outputi + 1                                                    
      end                                                                       
     db2i = db2i + 1                                                            
    end                                                                         
    conflict.outputi = ' '   /* an empty line after each output line */         
    outputi = outputi + 1                                                       
   end                                                                          
                                                                                
  if diff_col_id = 1 then                                      /* @01A*/        
   do                                                          /* @01A*/        
    conflict2_found = 1                                        /* @01A*/        
    conflict2.output2i = 'Catalog entry: ' ||,                 /* @01A*/        
                       center(oamcat.cati.id,14),              /* @01A*/        
                       left(oamcat.cati.name,44),              /* @01A*/        
                       center(oamcat.cati.sc,14),              /* @01A*/        
                       center(oamcat.cati.mc,17),              /* @01A*/        
                       center(oamcat.cati.group,14)            /* @01A*/        
    output2i = output2i + 1                                    /* @01A*/        
    conflict2.output2i = 'DB2 entry:     ' ||,                 /* @01A*/        
                       center(oamdb2.db2i.id,14),              /* @01A*/        
                       left(oamdb2.db2i.name,44),              /* @01A*/        
                       center(oamdb2.db2i.sc,14),              /* @01A*/        
                       center(oamdb2.db2i.mc,17),              /* @01A*/        
                       center(oamdb2.db2i.group,14)            /* @01A*/        
    output2i = output2i + 1                                    /* @01A*/        
    db2i = 1                                                   /* @01A*/        
    do while db2i <= db2_entry_num                             /* @01A*/        
     if oamcat.cati.id = oamdb2.db2i.id then                   /* @01A*/        
      do                                                       /* @01A*/        
       conflict2.output2i = '            (Collection entry ' ||,/*@01A*/        
                            oamdb2.db2i.name                   /* @01A*/        
       output2i = output2i + 1                                 /* @01A*/        
       conflict2.output2i = '             in DB2 with the '||, /* @01A*/        
                            ' same collection ID ' ||,         /* @01A*/        
                            oamdb2.db2i.id ||,                 /* @01A*/        
                            ' - SC: ' ||,                      /* @01A*/        
                            oamdb2.db2i.sc ||,                 /* @01A*/        
                            ' - MC: ' ||,                      /* @01A*/        
                            oamdb2.db2i.mc ||,                 /* @01A*/        
                            ' - SG: ' ||,                      /* @01A*/        
                            oamdb2.db2i.group ||,              /* @01A*/        
                            ') '                               /* @01A*/        
       output2i = output2i + 1                                 /* @01A*/        
      end                                                      /* @01A*/        
     db2i = db2i + 1                                           /* @01A*/        
    end                                                        /* @01A*/        
    conflict2.output2i = left('-',125,'-')                     /* @01A*/        
    output2i = output2i + 1                                    /* @01A*/        
   end                                                         /* @01A*/        
                                                                                
  found_in_db2 = 0                                                              
  diff_col_id = 0                                              /* @01A*/        
  cati = cati + 1                                                               
 end                                                                            
                                                                                
 if conflict_found = 0 & conflict2_found = 0 then              /* @01C*/        
  do                                                                            
   conflict.9 = 'No output found.'                                              
   say 'There is no conflict betweem OAM catalog entries and DB2' ||,           
   ' collection table.'                                        /* @01C*/        
   retcode = 4                                                                  
  end                                                                           
                                                                                
 else                                                          /* @01A*/        
  do                                                           /* @01A*/        
   if conflict_found = 1 then                                  /* @01A*/        
    do                                                         /* @01A*/        
     if batch_exec = 0 then                                    /* @05A*/        
      do                                                       /* @05A*/        
     say 'Enter the output data set name for missing' ||,      /* @01A*/        
         ' DB2 collection entries:'                            /* @01A*/        
     pull outputds                                                              
                                                                                
     ADDRESS TSO                                                                
     do while SYSDSN("'"outputds"'") = 'OK'                    /* @04C*/        
      say 'Dataset' outputds 'exists. Please enter a new one:' /* @04C*/        
      pull outputds                                            /* @04C*/        
     end                                                       /* @04C*/        
      end                                                                       
     "ALLOC F(outfile) DSN('"outputds"') SHR NEW SPACE(50,20),                  
     DSORG(PS) RECFM(F,B) LRECL(200) BLKSIZE(30000)"                            
     "EXECIO * DISKW outfile ( FINIS STEM conflict."                            
     "FREE F(outfile)"                                         /* @05A*/        
    end                                                        /* @01A*/        
   if conflict2_found = 1 then                                 /* @01A*/        
    do                                                         /* @01A*/        
     if batch_exec = 0 then                                    /* @05A*/        
      do                                                       /* @05A*/        
     say 'Enter the output data set name for same' ||,         /* @01A*/        
         ' collection name, different ID collection' ||,       /* @01A*/        
         ' entries:'                                           /* @01A*/        
     pull outputds2                                            /* @01A*/        
     ADDRESS TSO                                               /* @01A*/        
     do while SYSDSN("'"outputds2"'") = 'OK'                   /* @04C*/        
      say 'Dataset' outputds2 'exists. Please enter a new one:'/* @04C*/        
      pull outputds2                                           /* @04C*/        
     end                                                       /* @04C*/        
      end                                                      /* @05A*/        
    "ALLOC F(outfile2) DSN('"outputds2"') SHR NEW SPACE(50,20),/* @01A*/        
     DSORG(PS) RECFM(F,B) LRECL(200) BLKSIZE(30000)"           /* @01A*/        
     "EXECIO * DISKW outfile2 ( FINIS STEM conflict2."         /* @01A*/        
     "FREE F(outfile2)"                                        /* @01A*/        
    end                                                        /* @01A*/        
  end                                                          /* @01A*/        
 return                                                                         
                                                                                
/*-------------------------------------------------------------------*/         
/* Clean "1" at beginning of a line in data set                      */         
/*-------------------------------------------------------------------*/         
Clean1:                                                                         
 i = 1                                                                          
 do while i <= myfile.0        /* myfile.0 is the total line number  */         
  parse var myfile.i random1 real_line                                          
  if random1 = 1 then                                                           
   myfile.i = real_line                                                         
  i = i + 1                                                                     
 end                                                                            
return                                                                          
                                                                                
/*-------------------------------------------------------------------*/         
/* Error handling routine for bad SQL codes                          */         
/*-------------------------------------------------------------------*/         
SQLCA:                                                                          
 say "      SQLCODE = >"SQLCODE"<"                                              
 say "      SQLSTATE = >"SQLSTATE"<"                                            
 say "      SQLERRMC = >"SQLERRMC"<"                                            
 say "      SQLERRMC = >"SQLERRMC"<"                                            
return                                                                          
                                                                                
/*------------------------------------------------------------------*/          
/* Write Help Text for this exec (space holder for future)          */          
/*------------------------------------------------------------------*/          
Write_Help:                                                                     
say '                                                                  '        
                                                                                
exit                                                                            
