/* REXX */                                                                      
/*-------------------------------------------------------------------*/         
/*      NAME: CATSRCHI which was modeled after IGGCSIRX                         
 Description: This rexx exec is used to call the catalog search                 
              interface.                                                        
                                                                                
       Input: filter key and catalog field names (optional)                     
                                                                                
              The order of fieldnames is the way they are returned from         
              IGGCSI00 and thus the way they are parsed by the CATSRCHI         
              exec.  It is recommended that the VOLSER keyword be first         
              if it is included as the volser info is used several              
              times in reporting in the exec.                                   
                                                                                
      Variables:                                                                
      CAT() - use the catalog name to search for entries.  If not               
              specified Catalog Management will attempt to use the              
              high-level qualifier of CSIFILTK to locate an alias that          
              matches.  If an alias is found, the user catalog for that         
              alias will be searched; then, the master catalog will be          
              searched.  This is an optional parameter.                         
                                                                                
      TYPES() CSIDTYPS determines what type of catalog entries will be          
              returned. This is an optional parameter.                          
       Valid types for CSIDTYPS are:                                            
       Type      Description                                                    
       A         non-VSAM data set                    (Default)                 
       B         Generation data group                (Default)                 
       C         Cluster                              (Default)                 
       G         Alternate index                      (Default)                 
       H         Generation data set                  (Default)                 
       L         Tape volume catalog library entry    (Must be specified)       
       R         VSAM path                            (Default)                 
       U         User catalog connector entry         (Default)                 
       W         Tape volume catalog volume entry (Must be specified)           
       X         Alias                                (Default)                 
       Z         CAXWA                                (Must be specified)       
                                                                                
      All blanks for CSIDTYPS can be set and will get types A, B, C, G, H,      
      R, U, X. These are the catalog types. L and W must be explicitly          
      specified in order to get the Tape Volume Catalog entries.                
      Syntax of TYPES keyword: TYPES(A,B,C) TYPES(A B C) TYPES(A, B, C)         
                               TYPES(ABC)                                       
                                                                                
      WORK AREA() - sets the initial work area size passed to IGGCSI00          
                    This is an optional parameter that will default to          
                    999999. This parm can be specified with or without          
                    commas.                                                     
                                                                                
      INIT WORK() - sets the decimal value used to initialize the work          
                    area passed to IGGCSI00.  If the value passed is            
                    not in decimal form the exec will end with a                
                    condition code 22.                                          
                    This is an optional parameter that will default to          
                    x'00'                                                       
                                                                                
      Fieldnames() As documented in the manual Managing Catalogs, i.e.          
             %CATSRCHI KLMTEST1.KSDS02 FIELDNAMES(XHARBA XHURBA NOEXTNT)        
                                                                                
             %CATSRCHI KLMTEST1.KSDS02 FIELDNAMES(XHARBA XHURBA NOEXTNT) +      
             CAT(TEST.USERCAT)                                                  
                                                                                
             %CATSRCHI KLMTEST1.KSDS02 FIELDNAMES(XHARBA XHURBA NOEXTNT) +      
             TYPES(L,W)                                                         
                                                                                
      Output: Data returned from the catalog search interface for               
              which a parsing routine exists.                                   
                                                                                
      Condition Codes:                                                          
       20 - A filter key was not passed to the exec.                            
       21 - The catalog search interface did not find any matches based         
            on the filter key                                                   
       22 - The field passed in the INIT WORK() parm is not numeric.            
       xx - Any non-zero return code set by the catalog search interface        
                                                                                
  Written by: Kevin Miner 03/21/07                                              
                                                                                
  CHANGE ACTIVITY:                                                              
  03/28/07 JMR - Added parsing routine for AMDCIREC and generic                 
                 parsing routine to display any field                           
  03/29/07 JMR - Added parsing routine for AMDKEY and fixed problem             
                 with parsing for COMPIND                                       
  04/06/07 KLM - Modified Parse_Extent routine to handle multiple               
                 extents.                                                       
  04/09/07 KLM - Modified output format of Parse_Extent routine and             
                 added additional comments                                      
  06/11/07 KLM - Modified Parse_XHARBA and Parse_XHURBA to support              
                 repeating fields from IGGCSIOO.  Added a Parse_NOEXTNT         
                 subroutine.                                                    
  06/12/07 KLM - Added a Parse_XHURBADS routine and corrected a problem         
                 with the indexing in the Parse_NOEXTNT routine.                
  06/13/07 KLM - Corrected a problem with indexing in Parse_NOEXTNT and         
                 added volser info (if available) to the output for             
                 XHARBA, XHURBA and NOEXTNT.  Also flagged candidate            
                 volumes in the XHARBA, XHURBA and NOEXTNT parsing              
                 subroutines.  Added code to support IGGCSI00                   
                 reporting extents on multiple volumes.                         
  10/22/07 KLM - Added a subroutine to parse and print multiple entry           
                 data fields, i.e. volser(s), noextnt, extent, xharba           
                 and xhurba                                                     
  11/02/07 KLM - Changed the CSIOPTNS to a F so all length fields are           
                 returned as four bytes instead of two.  Added code             
                 to handle and report non-zero return codes from call           
                 to IGGCSI00.                                                   
  11/05/07 KLM - Added code to recognize when a call to IGGCSI does not         
                 return any matches based on the filter key.  When this         
                 happens the exec will end with a condition code 21.            
                 Changed variable names used in parsing IGGCSI00 data           
                 to match names used in z/OS DFSMS Managing Catalogs.           
                 Changed formatting of candidate volumes in XHARBA              
                 and XHURBA parsing subroutines to facilitate a                 
                 consistent output format when printing.                        
  03/17/08 KLM - Changed the code to not process XHARBADS and XHURBADS          
                 fields for the AIX portion of a VSAM data set.                 
  02/07/11 KLM - Added a Parse_NVSMATTR subroutine                              
  07/06/11 KLM - Added logic to support the CAT() and TYPE() parms as           
                 documented in the variable list above to support               
                 Tape Volume Catalog entries                                    
  01/10/12 KLM - Added logic in response to IBM PMR 06886,114,848 to            
                 test the csinoent bit in the csicflag on return from           
                 catalog to ensure data was passed for a catalog.               
  Mar 24, 2014 keminer - Added logic to redrive the CSI request with a          
                 larger work area if CAS indicated a larger work area           
                 csireqln is needed. OA44318 introduced a significant           
                 increase in the number of aliases that can be defined.         
  Mar 26, 2014 keminer - Added support for parms WORK AREA() INIT WORK()        
                 WORK AREA() is the size of the work area to be passed          
                 to IGGCSI00. INIT WORK() is the decimal value used to          
                 initialize the work area being passed to IGGCSI00.             
  Feb 25, 2015 keminer - Added logic to support TYPE Z caxwa and a              
                 subroutine to parse and print fields returned                  
  Jun 18, 2015 keminer - Added logic to support writing out CSI                 
                 information even after an error is encountered.                
  Jun 24, 2015 keminer - Added additional logic to write out field              
                 keys and data only if data was returned.                       
                                                                                
*/                                                                              
/*-------------------------------------------------------------------*/         
 arg parmin                          /* Save any parms passed to exec*/         
 upper parmin                        /* Convert parms to uppercase   */         
 Call Initialize                     /* Initialize exec variables    */         
 Call Call_IGGCSI00                  /* Call catalog search interface*/         
                                                                                
 exit(CSIRC)                         /* Exit with CSI return code    */         
                                                                                
/**************************** Subroutines ****************************/         
/*-------------------------------------------------------------------*/         
/* Initialize the variables used by this exec                        */         
/*-------------------------------------------------------------------*/         
Initialize:                                                                     
 numeric digits 12                                                              
 if words(parmin) > 0 then           /* Were parms passed to exec?   */         
  do                                 /* Yes,                         */         
   If pos('HELP',parmin) > 0 then Call Write_Help                               
   key = word(parmin,1)              /* Set filter key to 1st word   */         
  end                                                                           
 else                                                                           
  if SYSVAR(SYSENV) = 'FORE' then    /* if running in foreground     */         
   do                                                                           
    SAY 'ENTER FILTER KEY'           /* ASK FOR FILTER KEY           */         
    PULL KEY                         /* GET FILTER KEY               */         
   end                                                                          
  else                                                                          
   do                                                                           
    say ' When run as a batch job this exec requires an input parm'             
    say ' of the filter key to be passed to IGGCSI00.             '             
    exitrc = 20                                                                 
    Call Write_Return_Codes                                                     
   end                                                                          
                                                                                
 dsname    = ''                      /* Save dsn returned by IGGCSI00*/         
 exitrc    = 0                       /* Set exit rc to zero          */         
 highrc    = 0                       /* Save highest CSI rc          */         
 noextnt.0 = 0                       /* Set array entries to zero    */         
 vollist.0 = 0                       /* Set array entries to zero    */         
 volcount = 0                        /* Set variable to zero         */         
 vvr_extent_count = 0                /* Set variable to zero         */         
                                                                                
 /* Look for a user specified work area size to pass to IGGCSI00     */         
 If pos('WORK AREA(',parmin) > 1 then                                           
  do                                                                            
   parse var parmin 'WORK AREA('worklen')'                                      
   worklen = translate(worklen,' ',',') /*translate ',' to blanks    */         
   worklen = space(worklen,0) /* remove any blanks                   */         
  end                                                                           
 else worklen = 999999                                                          
                                                                                
 /* Look for a user specified value to initialize work area passed to           
    IGGCSI00                                                         */         
 If pos('INIT WORK(',parmin) > 1 then                                           
  do                                                                            
   parse var parmin 'INIT WORK('initwork')'                                     
   initwork = translate(initwork,' ',',') /*translate ',' to blanks  */         
   initwork = space(initwork,0) /* remove any blanks                 */         
   if datatype(initwork) <> 'NUM' then /* If field passed not numeric*/         
    do                                                                          
     exitrc = '22'                                                              
     Call Write_Return_Codes                                                    
    end                                                                         
   initwork = d2x(initwork)     /* Convert to hex format             */         
  end                                                                           
 else initwork = d2x('00')                                                      
                                                                                
/*  dwork = '000F423F'X || COPIES('00'X,worklen-5)                   */         
 xworklen = d2x(worklen,8)                                                      
 interpret "dwork = '"||xworklen||"'X"                                          
 /* dwork = dwork|| COPIES('00'X,worklen-5) */                                  
 interpret "dwork = dwork|| COPIES('"initwork"'X,worklen-5)"                    
                                                                                
/* Set default CSI parameter list                                    */         
 MODRSNRC = SUBSTR(' ',1,4)          /* CLEAR MODULE/RETURN/REASON   */         
 CSIFILTK = SUBSTR(KEY,1,44)         /* MOVE FILTER KEY INTO LIST    */         
 CSICATNM = SUBSTR(' ',1,44)         /* CLEAR CATALOG NAME           */         
 CSIRESNM = SUBSTR(' ',1,44)         /* CLEAR RESUME NAME            */         
 CSIDTYPS = SUBSTR(' ',1,16)         /* CLEAR ENTRY TYPES            */         
 CSICLDI  = SUBSTR('Y',1,1)          /* INDICATE DATA AND INDEX      */         
 CSIRESUM = SUBSTR(' ',1,1)          /* CLEAR RESUME FLAG            */         
 CSIS1CAT = SUBSTR(' ',1,1)          /* INDICATE SEARCH > 1 CATALOGS */         
 CSIOPTNS = SUBSTR('F',1,1)          /* An F entry = fullword lengths*/         
                                     /* anything else use halfwords  */         
 CSINUMEN = '0006'X                  /*   INIT NUMBER OF FIELDS      */         
 CSIFLD1    = ''                     /*   INIT FIELD 1 FOR VOLSERS   */         
 CSIFLD1    = CSIFLD1||SUBSTR('VOLSER',1,8)                                     
 CSIFLD1    = CSIFLD1||SUBSTR('ATTR1 ',1,8)                                     
 CSIFLD1    = CSIFLD1||SUBSTR('XHARBADS',1,8)                                   
 CSIFLD1    = CSIFLD1||SUBSTR('XACIFLAG',1,8)                                   
 CSIFLD1    = CSIFLD1||SUBSTR('EXTENT ',1,8)                                    
 CSIFLD1    = CSIFLD1||SUBSTR('XHURBA',1,8)                                     
 fieldkey_count = c2d(CSINUMEN)    /* Get count of field keys used   */         
                                                                                
 /* Replace default field names with any passed by the user          */         
 If pos('FIELDNAMES(',parmin) > 1 then                                          
  do                                                                            
   /* Get list of field names passed by the user                     */         
   parse var parmin 'FIELDNAMES(' user_fieldnames ')' junk                      
   /* Place a single space between each user specified field name    */         
   user_fieldnames = space(user_fieldnames,1)                                   
   CSIFLD1 = user_fieldnames                                                    
                                                                                
   /* Get a count of the field names passed by the user              */         
   user_fieldname_count = words(user_fieldnames)                                
   fieldkey_count = user_fieldname_count /* Save count of field keys */         
                                                                                
   /* Replace the count in the IGGCSI00 parameter list               */         
   CSINUMEN = d2c(user_fieldname_count,2)  /* Convert to hex format  */         
   CSIFLD1 = ''                      /* Clear the field name list    */         
   /* Rebuild fieldname list padding each field to 8 characters      */         
   do ufn = 1 to user_fieldname_count                                           
    CSIFLD1 = CSIFLD1||LEFT(WORD(user_fieldnames,ufn),8)                        
   end                                                                          
  end                                                                           
                                                                                
 /* CSI will use the catalog name supplied in CSICATNM to search for            
    entries if CSICATNM is not blanks. If CSICATNM is blanks,                   
    Catalog Management will attempt to use the high-level qualifier             
    of CSIFILTK to locate an alias that matches.  If an alias is                
    found, the user catalog for that alias will be searched; then,              
    the master catalog will be searched.                                        
*/                                                                              
 If pos('CAT(',parmin) > 1 then  /* Was a catalog name passed?       */         
  do                                                                            
   parse var parmin 'CAT(' CSICATNM ')' junk                                    
   CSICATNM = left(CSICATNM,44,' ')                                             
  end                                                                           
                                                                                
/* CSIDTYPS determines what type of catalog entries will be returned.           
   Valid types for CSIDTYPS are:                                                
   Type      Description                                                        
   A         non-VSAM data set                    (Default)                     
   B         Generation data group                (Default)                     
   C         Cluster                              (Default)                     
   G         Alternate index                      (Default)                     
   H         Generation data set                  (Default)                     
   L         Tape volume catalog library entry    (Must be specified)           
   R         VSAM path                            (Default)                     
   U         User catalog connector entry         (Default)                     
   W         Tape volume catalog volume entry (Must be specified)               
   X         Alias                                (Default)                     
                                                                                
  All blanks for CSIDTYPS can be set and will get types A, B, C, G, H,          
  R, U, X. These are the catalog types. L and W must be explicitly              
  specified in order to get the Tape Volume Catalog entries.                    
                                                                                
*/                                                                              
 If pos('TYPES(',parmin) > 1 then /*Restrict catalog entries by type?*/         
  do                                                                            
   parse var parmin 'TYPES(' CSIDTYPS ')' junk /*User requested types*/         
   CSIDTYPS = translate(CSIDTYPS,' ',',') /*translate ',' to blanks  */         
   CSIDTYPS = space(CSIDTYPS,0) /* remove any blanks                 */         
   CSIDTYPS = left(CSIDTYPS,16,' ')  /* Set for IGGCSI00 call        */         
  end                                                                           
                                                                                
 /* Build the selection criteria fields part of parameter list       */         
 CSIOPTS  = CSICLDI || CSIRESUM || CSIS1CAT || CSIOPTNS                         
 CSIFIELD = CSIFILTK || CSICATNM || CSIRESNM || CSIDTYPS || CSIOPTS             
 CSIFIELD = CSIFIELD || CSINUMEN || CSIFLD1                                     
                                                                                
 /* Initialize and build work area output part of parameter list     */         
 csicnameT = SUBSTR(' ',1,44)                                                   
 PREVNAME = ''                     /* Clear previous name variable   */         
 RESUME   = 'Y'                                                                 
                                                                                
return                                                                          
                                                                                
/*-------------------------------------------------------------------*/         
/* This routine will do the actual call to IGGCSI00 and parse the               
   resulting output.                                                 */         
/*-------------------------------------------------------------------*/         
Call_IGGCSI00:                                                                  
 field_offset = 1                  /* Offset into list of field kys*/           
 fieldname_list = ''               /* Blank separated field key lst*/           
 /* Create an array of the list of field keys passed to IGGCSI00   */           
 do fkc = 1 to fieldkey_count                                                   
  CSIFLD1.fkc = substr(CSIFLD1,field_offset,8)                                  
  /* Add to list of field names after removing any extra blanks    */           
  fieldname_list = fieldname_list space(CSIFLD1.fkc,0)                          
  if fkc < fieldkey_count then     /* Increment to next field key  */           
    field_offset = field_offset + 8                                             
 end                                                                            
                                                                                
 say ' '                                                                        
 say ' Catalog Search Interface is being called with filter key:' key           
 say ' and catalog field name(s):' fieldname_list                               
 If length(strip(CSICATNM)) > 0 then                                            
 say ' Using catalog:' CSICATNM                                                 
 If length(strip(CSIDTYPS)) > 0 then                                            
 say ' Requesting catalog entries:' CSIDTYPS                                    
 say ' '                                                                        
                                                                                
/* Set up loop for resume (if a resume is ncessary)                  */         
Do While RESUME = 'Y'                                                           
 /* Issue link to catalog generic filter interface                   */         
 ADDRESS LINKPGM 'IGGCSI00  MODRSNRC  CSIFIELD  dwork'                          
 CSIRC = RC                          /* Save return code from call   */         
 /* rwa is acronym for returned work area.                           */         
 rwaindex=0                         /*Start loc of returned work area*/         
 If CSIRC > 0 THEN                  /* Test for non-zero return code*/          
  Do                                                                            
   say '  '                                                                     
   say ' The Catalog Search Interface, IGGCSI00 ended with a condition'         
   say ' code of 'csirc                                                         
   moduleid = substr(MODRSNRC,1,2)  /* Module id is the first 2 bytes*/         
   reasoncode = c2x(substr(MODRSNRC,3,1)) /*reason code is byte 3    */         
   returncode = c2x(substr(MODRSNRC,4,1)) /*return code is byte 4    */         
   say ' Module Id= 'moduleid                                                   
   say " Return Code = X('"returncode"')" x2d(returncode)                       
   say " Reason Code = X('"reasoncode"')" x2d(reasoncode)                       
   say '  '                                                                     
  End                                                                           
 RESUME = SUBSTR(CSIFIELD,150,1)    /* GET RESUME FLAG FOR NEXT LOOP */         
 csiusdln = c2d(substr(dwork,9,4))  /* Get amount of work area used  */         
                                                                                
/* Process data returned in work area                                */         
 csiusrln = c2d(substr(dwork,1,4))  /* Get work area total length    */         
 csireqln = c2d(substr(dwork,5,4))  /* Min required work area        */         
 csiusdln = c2d(substr(dwork,9,4))  /* Get amount of work area used  */         
 csinumfd = c2d(substr(dwork,13,2)) /* Number of field names plus 1  */         
 csicflg  = x2b(c2x(substr(dwork,15,1)))                                        
 CSICTYPE = c2x(SUBSTR(dwork,+16,1)) /* Catalog type x'F0'    */                
 csicname=SUBSTR(dwork,17,44)       /* Get catalog name from area    */         
 rwaindex=65                        /*Start loc of returned work area*/         
                                                                                
 /* The next 9 lines were commented out as these are for testing                
 say ' csiusrln (Total length of return work area) =' csiusrln                  
 say ' csireqln (Minimum required work area      ) =' csireqln                  
 say ' csiusdln (Total length of work area used  ) =' csiusdln                  
 say ' csinumfd+1 =' csinumfd                                                   
 say ' csicflg    =' csicflg                                                    
 say ' csictype   =' csictype                                                   
 say ' csicname   =' csicname                                                   
 say ' rwaindex   = 'rwaindex                                                   
 say '  '                                                                       
    */                                                                          
                                                                                
 /* The size of the work area table is dependent on CSIOPTNS         */         
 if CSIOPTNS = ' ' then                                                         
  temp_worka = SUBSTR(dwork,rwaindex,50)                                        
 else                                                                           
  temp_worka = SUBSTR(dwork,rwaindex,54)                                        
                                                                                
 Do while rwaindex < CSIUSDLN       /* DO UNTIL ALL DATA IS PROCESSED*/         
  If SUBSTR(dwork,rwaindex+1,1) = '0' then /*Is this a catalog entry?*/         
   do                                /* process catalog entry        */         
    csicname=SUBSTR(dwork,rwaindex+2,44) /*Get catalog name from area*/         
    /* CSICFLG contains information returned for each catalog        */         
    csicflag = x2b(c2x(substr(dwork,rwaindex,1))) /*Convert to binary*/         
    /* csinoent is a bit in csicflag when on there are no entries for*/         
    /* this catalog                                                  */         
    csinoent = substr(csicflag,2,1)                                             
    /* Catalog entries are 50 bytes, index to next entry in work area*/         
    rwaindex = rwaindex + 50                                                    
    if csinoent = 1 then iterate   /* 1 = no entries for this catalog*/         
   end                                                                          
                                                                                
   csieflag = x2b(c2x(substr(dwork,rwaindex,1)))                                
   csienter = substr(csieflag,2,1) /* 1 = Error is set for this entry*/         
   csiedata = substr(csieflag,3,1) /* 1=Data returned for this entry */         
   csietype = SUBSTR(dwork,rwaindex+1,1)  /* Get entry type          */         
   csiename = SUBSTR(dwork,rwaindex+2,44) /* Get entry name          */         
                                                                                
   /* If this entry is flagged in error, write data and save rc      */         
   if csienter = 1 then                                                         
   do                                                                           
    csieretm = SUBSTR(dwork,rwaindex+46,2)  /* Entry return module ID*/         
    csieretr = c2d(SUBSTR(dwork,rwaindex+48,1)) /*Entry retrn rsn cd */         
    csieretc = c2d(SUBSTR(dwork,rwaindex+49,1)) /*Entry return code  */         
    say '    '                                                                  
    say '  CSIEFLAG......'csieflag                                              
    say '  CSIETYPE = 'csietype                                                 
    say '  CSIENAME = 'csiename                                                 
    dr= '  CSIETETM (Entry return module ID            ) = '                    
    say dr'IGG0CL'csieretm                                                      
    say '  CSIERETC (Entry return code                 ) = 'csieretc            
    say '  CSIERETR (Entry return reason code          ) = 'csieretr            
    say '  See IDC3009I' csieretc 'then look for reason code 'csieretr          
    exitrc = csieretc                                                           
    if highrc < csieretc  then highrc = csieretc /* Save highest rc  */         
/*  Call Write_Return_Codes */                                                  
   end                                                                          
                                                                                
  if csiedata = 1 then                /* If data was returned        */         
   do                                                                           
   if CSIOPTNS = ' ' then                                                       
    do                                                                          
     csitotln=c2d(SUBSTR(dwork,rwaindex+46,2))                                  
     CSILENF1=c2d(SUBSTR(dwork,rwaindex+50,2))                                  
     rwaentry_len = 46                /* Offset for len of rwa entry */         
     len_offset   = 4                  /* Offset of field len in rwa  */        
    end                                                                         
   else                                                                         
    do                                                                          
     csitotln=c2d(SUBSTR(dwork,rwaindex+46,4))                                  
     CSILENF1=c2d(SUBSTR(dwork,rwaindex+54,4))                                  
     rwaentry_len = 46                /* Offset for len of rwa entry */         
     len_offset   = 4                  /* Offset of field len in rwa */         
    end                                                                         
   end                                                                          
  else                                                                          
   do   /* When there is an error                                    */         
    csitotln = 4    /* Set total length of returned info to 4        */         
    CSILENF1 = 0    /* Set length of first field to zero             */         
                                                                                
    /*  This ws used during testing to see actual data returned                 
    queue  dwork                                                                
    queue ''                                                                    
    "Alloc dd(iggdata) da(iggdata) shr reuse"                                   
    "execio * diskw iggdata (finis"                                             
    "free dd(iggdata)"                                                          
    say '<<<<<<<<<<<< Wrote IGGDATA >>>>>>>>>>>>>>>>>>>>'                       
     */                                                                         
                                                                                
   end                                                                          
                                                                                
  If rwaindex < CSIUSDLN THEN        /* IF STILL MORE DATA       @02A*/         
   Do                                /* CONTINUE WITH NEXT ENTRY @02A*/         
    say '  '                                                                    
    /* Calculate location of next entry in the returned work area    */         
    next_rwa_entry = rwaindex + rwaentry_len /*Add len of table entry*/         
    next_rwa_entry = next_rwa_entry + csitotln /* Plus data length   */         
    /* Calculate length of data portion of this entry                */         
    length_temp_worka = next_rwa_entry - rwaindex                               
                                                                                
    /* Copy the data for this return work area entry to simplify     */         
    /* parsing the data                                              */         
    temp_worka = SUBSTR(dwork,rwaindex,length_temp_worka)                       
                                                                                
   /* Assign entry type name                                         */         
   ENTRY_TYPE = SUBSTR(temp_worka,2,1) /* Get entry type             */         
   Select                              /* Assign name for this entry */         
    WHEN ENTRY_TYPE = '0' THEN DTYPE = 'CATALOG '                               
    WHEN ENTRY_TYPE = 'C' THEN DTYPE = 'CLUSTER '                               
    WHEN ENTRY_TYPE = 'D' THEN DTYPE = 'DATA    '                               
    WHEN ENTRY_TYPE = 'I' THEN DTYPE = 'INDEX   '                               
    WHEN ENTRY_TYPE = 'A' THEN DTYPE = 'NONVSAM '                               
    WHEN ENTRY_TYPE = 'H' THEN DTYPE = 'GDS     '                               
    WHEN ENTRY_TYPE = 'B' THEN DTYPE = 'GDG     '                               
    WHEN ENTRY_TYPE = 'R' THEN DTYPE = 'PATH    '                               
    WHEN ENTRY_TYPE = 'G' THEN DTYPE = 'AIX     '                               
    WHEN ENTRY_TYPE = 'X' THEN DTYPE = 'ALIAS   '                               
    WHEN ENTRY_TYPE = 'U' THEN DTYPE = 'UCAT    '                               
    WHEN ENTRY_TYPE = 'L' THEN DTYPE = 'ATLLIB  '                               
    WHEN ENTRY_TYPE = 'W' THEN DTYPE = 'ATLVOL  '                               
    WHEN ENTRY_TYPE = 'Z' THEN DTYPE = 'CAXWA   '                               
    Otherwise DTYPE = ''                                                        
   End                                                                          
                                                                                
   If DTYPE = '' then iterate        /* Unknown entry type so skip it*/         
  /* Have name and type, get volser info.  if we found a valid type  */         
  /* for this catalog, now print the catalog name (first entry only).*/         
   If csicname ^= csicnameT THEN /* IF RESUME NAME MAY ALREADY @02C  */         
    Do                         /*    BE PRINTED                      */         
     SAY 'Catalog ' csicname   /* IF NOT, PRINT IT             @02C  */         
     csicnameT = csicname                                    /*@02C  */         
    End                                                      /*@02C  */         
   /* If all that was returned was a catalog entry then iterate      */         
   If DTYPE = 'CATALOG ' then iterate                                           
   rwaentry_len = 46                                                            
                                                                                
   /* Create an array of the length of each data field returned by   */         
   /* IGGCSI00 for each key                                          */         
   /* The offset of returned length data is dependent on CSIOPTNS    */         
   if CSIOPTNS = ' ' then len_offset = len_offset + 1                           
   else len_offset = len_offset + 5                                             
                                                                                
   /* Only write out field keys and data if data was returned        */         
   do fkc = 1 to fieldkey_count while csiedata = 1                              
   if CSIOPTNS = ' ' then                                                       
    do                                                                          
     CSILENF.fkc = C2D(SUBSTR(temp_worka,rwaentry_len+len_offset,2))            
     len_offset = len_offset + 2     /* Increment to next field key  */         
    end                                                                         
   else                                                                         
    do                                                                          
     CSILENF.fkc = C2D(SUBSTR(temp_worka,rwaentry_len+len_offset,4))            
     len_offset = len_offset + 4     /* Increment to next field key  */         
    end                                                                         
   end                                                                          
                                                                                
   /* Find the start location for data in this entry                 */         
   index2_rwadata = rwaentry_len+len_offset                                     
   /* Set dsname only for valid entry types                         */          
   dsname = SUBSTR(temp_worka,3,44) /* GET ENTRY NAME               */          
   if csiedata > 0 then say ' 'dtype dsname                                     
                                                                                
   /* Process returned data for each field key                       */         
   do fkc = 1 to fieldkey_count while csiedata > 0                              
    Select                                                                      
     When CSIFLD1.fkc = 'VOLSER' then Call Parse_Volser_List                    
     When CSIFLD1.fkc = 'XHARBA' then Call Parse_XHARBA                         
     When CSIFLD1.fkc = 'XHARBADS' then Call Parse_XHARBADS                     
     When CSIFLD1.fkc = 'XHURBADS' then Call Parse_XHURBADS                     
     When CSIFLD1.fkc = 'XHURBA' then Call Parse_XHURBA                         
     When CSIFLD1.fkc = 'EXTENT' then Call Parse_EXTENT                         
     When CSIFLD1.fkc = 'NOEXTNT' then Call Parse_NOEXTNT                       
     When CSIFLD1.fkc = 'XACIFLAG' then Call Parse_XACIFLAG                     
     When CSIFLD1.fkc = 'COMPIND' then Call Parse_COMPIND                       
     When CSIFLD1.fkc = 'AMDCIREC' then Call Parse_AMDCIREC                     
     When CSIFLD1.fkc = 'AMDKEY' then Call Parse_AMDKEY                         
     When CSIFLD1.fkc = 'NVSMATTR' then Call Parse_NVSMATTR                     
     When CSIFLD1.fkc = 'LCBCONID' then Call Parse_Library                      
     When CSIFLD1.fkc = 'LCBDESCR' then Call Parse_Library                      
     When CSIFLD1.fkc = 'LCBDEVTP' then Call Parse_Library                      
     When CSIFLD1.fkc = 'LCBLIBID' then Call Parse_Library                      
     When CSIFLD1.fkc = 'VCBCHKPT' then Call Parse_Library                      
     When CSIFLD1.fkc = 'VCBCRDT' then Call Parse_Library                       
     When CSIFLD1.fkc = 'VCBEDATE' then Call Parse_Library                      
     When CSIFLD1.fkc = 'VCBLIBNM' then Call Parse_Library                      
     When CSIFLD1.fkc = 'VCBLOC' then Call Parse_Library                        
     When CSIFLD1.fkc = 'VCBMOUNT' then Call Parse_Library                      
     When CSIFLD1.fkc = 'VCBOWNER' then Call Parse_Library                      
     When CSIFLD1.fkc = 'VCBSGRP' then Call Parse_Library                       
     When CSIFLD1.fkc = 'VCBSHELF' then Call Parse_Library                      
     When CSIFLD1.fkc = 'VCBWRITE' then Call Parse_Library                      
     When CSIFLD1.fkc = 'ASSOCSYM' then Call Parse_ASSOCSYM                     
     When CSIFLD1.fkc = 'CATFLAGS' then Call Parse_CatFlags                     
     Otherwise                                                                  
      do                                                                        
       Call Parse_Generic                                                       
      end                                                                       
    end                                                                         
   end                                                                          
   rwaindex = next_rwa_entry   /* Point to next work area entry      */         
  END                          /*                                    */         
 END                                                                            
 if resume = 'y' &,            /* if we've tried this entry @01a     */         
  PREVNAME = dsname then       /*  TWICE, WE'VE GOT TO QUIT      @01A*/         
  DO                           /*                               @01A */         
   say strip(dsname) 'Cannot be processed with the work area size ',            
        'provided - you must increase the work area and retry'                  
   RETURN                      /*                                @01A*/         
  end                          /*                                @01a*/         
 PREVNAME = dsname             /*      SAVE FOR NEXT ITERATION   @01A*/         
 /*If IGGCSI req len (csireqln) is greater that the user specified              
   length (csiusrln) and csireqln GT zero the CAS determinged that              
   a larger work area is needed so redo the work area and size passed           
   to IGGCSI.  The RESUME flag is expected to be 'Y' which will                 
   redrive the ADDRESS LINKPGM 'IGGCSI00  MODRSNRC  CSIFIELD  dwork'            
   call to CSI at the top of this loop.                              */         
 if csireqln > csiusrln & csireqln > 0 then                                     
  do                                                                            
   xworklen = d2x(csireqln,8)  /* Min required work area from CAS    */         
   interpret "dwork = '"||xworklen||"'X" /* Convert to hex format    */         
   dwork = dwork|| COPIES('00'X,csireqln-5) /* Pad with hex zeros    */         
  end                                                                           
END                                                                             
                                                                                
if dsname ='' then                                                              
 do                                                                             
  say '   '                                                                     
  say '  IGGCSI00 did not find any matches with filter key:' key                
  say '   '                                                                     
  exitrc = 21                                                                   
  Call Write_Return_Codes                                                       
 end                                                                            
return                                                                          
                                                                                
/*------------------------------------------------------------------*/          
/* Process the list of volsers passed back in the return work area .*/          
/*------------------------------------------------------------------*/          
Parse_Volser_List:                                                              
 if CSILENF.fkc = 0 then return  /* No data returned for this key   */          
                                                                                
 pos_of_volsers = index2_rwadata /* Position on volser data         */          
 NUMVOL = CSILENF.fkc/6          /*How many volsers are listed?     */          
 volser_list = ''               /* Clear volser list                */          
 Do vl = 1 TO NUMVOL            /* Copy volser from rwa to vol list */          
  volser = SUBSTR(temp_worka,pos_of_volsers,6)                                  
  VOLSER_LIST = VOLSER_LIST volser                                              
  vollist.vl= volser            /* Add to volser list array         */          
  vollist.0 = numvol            /* Save number of entries in array  */          
  pos_of_volsers = pos_of_volsers + 6 /* pos of next volser         */          
 End                                                                            
                                                                                
 /* Set up variables to print the list of associated volsers         */         
 words_per_entry = 1           /* Number of words in list per entry  */         
 left_justify = 0              /* Left justify each entry            */         
 entries_per_line = 8          /* Number of entries per print line   */         
 first_line_prefix = ' On volser(s)'                                            
 other_line_prefix = '             '                                            
 list_2parse = volser_list                                                      
 Call Print_List                                                                
                                                                                
 index2_rwadata = pos_of_volsers     /* Save index into rwa data    */          
                                                                                
return                                                                          
                                                                                
/*------------------------------------------------------------------*/          
/* Process the XHARBA data                                          */          
/*                                                                              
 trace ?i                                                                       
 queue  temp_worka                                                              
 queue ''                                                                       
 "execio * diskw iggdata (finis"                                                
 SAY 'index2_rwadata = ' index2_rwadata                                         
 SAY 'CSILENF.fkc    = ' CSILENF.fkc                                            
 */                                                                             
/*------------------------------------------------------------------*/          
Parse_XHARBA:                                                                   
 numeric digits 12             /* Set processing to 12 digits        */         
 process_flag = 'y'                                                             
 If dtype <> 'DATA',           /* Is this ds valid for RBA?          */         
  & dtype <> 'INDEX',                                                           
  & dtype <> 'AIX' then process_flag = 'n' /* Turn off process flag  */         
 if CSILENF.fkc = 0,           /* No data returned for this key      */         
  | process_flag = 'n' then    /* No data returned for this key      */         
  do                                                                            
   say '  XHARBA = no data'                                                     
  /*Add data length for this field name to the return work area index*/         
   index2_rwadata = index2_rwadata + CSILENF.fkc                                
   return                                                                       
  end                                                                           
                                                                                
 pos_of_xharba = index2_rwadata /* Position of xhrba data            */         
 numvol = CSILENF.fkc/8         /* How many volsers are listed?      */         
 xharba_list = ''               /* Clear xharba list                 */         
 Do vl = 1 to numvol            /*                                   */         
  /* Copy xhurba to a temp work area                                 */         
  temp_field = substr(temp_worka,pos_of_xharba,8)                               
  temp_field = C2X(temp_field)  /* Convert to hex                    */         
  /* When IGGCSI00 returns a field of x'F's the field is equal to 0  */         
  if pos('FFFFFFFF',temp_field) > 0 then xharba_data= '0-Candidate'             
  else                                                                          
   xharba_data= C2D(SUBSTR(temp_worka,pos_of_xharba,8))                         
                                                                                
  pos_of_xharba = pos_of_xharba + 8 /* pos of next xharba entry    */           
  /* Build list of xharba                                          */           
  if vollist.0 > 0 then                                                         
   xharba_list=xharba_list 'Volume' vollist.vl 'XHARBA = 'xharba_data           
  else                                                                          
   xharba_list=xharba_list 'Volume' vl 'XHARBA = 'xharba_data                   
 End                                                                            
                                                                                
 /* Set up variables to print the XHARBA for each volser             */         
 words_per_entry = 5           /* Number of words in list per entry  */         
 left_justify = 38             /* Left justify each entry            */         
 entries_per_line = 2          /* Number of entries per print line   */         
 first_line_prefix = ''                                                         
 other_line_prefix = ''                                                         
 list_2parse = xharba_list     /* Set list to be parsed              */         
 Call Print_List                                                                
                                                                                
 index2_rwadata = pos_of_xharba      /* Save index into rwa data    */          
return                                                                          
                                                                                
/*------------------------------------------------------------------*/          
/* Process the XHARBADS data                                        */          
/*------------------------------------------------------------------*/          
Parse_XHARBADS:                                                                 
 numeric digits 12             /* Set processing to 12 digits        */         
 process_flag = 'y'                                                             
 If dtype <> 'DATA',           /* Is this ds valid for RBA?          */         
  & dtype <> 'INDEX' then process_flag = 'n' /* Turn off process flag*/         
 if CSILENF.fkc = 0,           /* No data returned for this key      */         
  | process_flag = 'n' then    /* No data returned for this key      */         
  do                                                                            
   say '  XHARBADS = no data'                                                   
 /*Add data length for this field name to the return work area index */         
   index2_rwadata = index2_rwadata + CSILENF.fkc                                
   return                                                                       
  end                                                                           
                                                                                
 XHARBADS= C2D(SUBSTR(temp_worka,index2_rwadata,CSILENF.fkc))                   
 say '  XHARBADS =' XHARBADS                                                    
 /*Add data length for this field name to the return work area index */         
 index2_rwadata = index2_rwadata + CSILENF.fkc                                  
return                                                                          
                                                                                
/*------------------------------------------------------------------*/          
/* Process the XHURBADS data                                        */          
/*------------------------------------------------------------------*/          
Parse_XHURBADS:                                                                 
 numeric digits 12             /* Set processing to 12 digits        */         
 process_flag = 'y'                                                             
 If dtype <> 'DATA',           /* Is this ds valid for RBA?          */         
  & dtype <> 'INDEX'then process_flag = 'n' /* Turn off process flag */         
 if CSILENF.fkc = 0,           /* No data returned for this key      */         
  | process_flag = 'n' then    /* No data returned for this key      */         
  do                                                                            
   say '  XHURBADS = no data'                                                   
 /*Add data length for this field name to the return work area index */         
   index2_rwadata = index2_rwadata + CSILENF.fkc                                
   return                                                                       
  end                                                                           
                                                                                
 XHURBADS= C2D(SUBSTR(temp_worka,index2_rwadata,CSILENF.fkc))                   
 say '  XHURBADS =' XHURBADS                                                    
 /*Add data length for this field name to the return work area index */         
 index2_rwadata = index2_rwadata + CSILENF.fkc                                  
return                                                                          
                                                                                
/*------------------------------------------------------------------*/          
/* Process the XHURBA data                                          */          
/*------------------------------------------------------------------*/          
Parse_XHURBA:                                                                   
 numeric digits 12             /* Set processing to 12 digits        */         
 process_flag = 'y'                                                             
 If dtype <> 'DATA',           /* Is this ds valid for RBA?          */         
  & dtype <> 'INDEX',                                                           
  & dtype <> 'AIX' then process_flag = 'n' /* Turn off process flag  */         
 if CSILENF.fkc = 0,           /* No data returned for this key      */         
  | process_flag = 'n' then    /* No data returned for this key      */         
  do                                                                            
   say '  XHURBA = no data'                                                     
  /*Add data length for this field name to the return work area index*/         
   index2_rwadata = index2_rwadata + CSILENF.fkc                                
   return                                                                       
  end                                                                           
                                                                                
 pos_of_xhurba = index2_rwadata /* Position of xhrba data            */         
 numvol = CSILENF.fkc/8         /* How many volsers are listed?      */         
 xhurba_list = ''               /* Clear xhurba list                 */         
 Do vl = 1 to numvol            /*                                   */         
  /* Copy xhurba to a temp work area                                 */         
  temp_field = substr(temp_worka,pos_of_xhurba,8)                               
  temp_field = C2X(temp_field)  /* Convert to hex                    */         
  /* When IGGCSI00 returns a field of x'F's the field is equal to 0  */         
  if pos('FFFFFFFF',temp_field) > 0 then xhurba_data= '0-Candidate'             
  else                                                                          
   xhurba_data= C2D(SUBSTR(temp_worka,pos_of_xhurba,8))                         
                                                                                
  pos_of_xhurba = pos_of_xhurba + 8 /* pos of next xhurba entry    */           
  if vollist.0 > 0 then                                                         
   xhurba_list=xhurba_list 'Volume' vollist.vl 'XHURBA = 'xhurba_data           
  else                                                                          
   xhurba_list=xhurba_list 'Volume' vl 'XHURBA = 'xhurba_data                   
 End                                                                            
                                                                                
 /* Set up variables to print the XHARBA for each volser             */         
 words_per_entry = 5           /* Number of words in list per entry  */         
 left_justify = 38             /* Left justify each entry            */         
 entries_per_line = 2          /* Number of entries per print line   */         
 first_line_prefix = ''                                                         
 other_line_prefix = ''                                                         
 list_2parse = xhurba_list     /* Set list to be parsed              */         
 Call Print_List                                                                
                                                                                
 index2_rwadata = pos_of_xhurba      /* Save index into rwa data    */          
return                                                                          
                                                                                
/*------------------------------------------------------------------*/          
/* Process the NOEXTNT data                                         */          
/*------------------------------------------------------------------*/          
Parse_NOEXTNT:                                                                  
 numeric digits 12             /* Set processing to 12 digits        */         
 process_flag = 'y'                                                             
 If dtype <> 'DATA',           /* Is this ds valid for RBA?          */         
  & dtype <> 'INDEX',                                                           
  & dtype <> 'AIX' then process_flag = 'n' /* Turn off process flag  */         
 if CSILENF.fkc = 0,           /* No data returned for this key      */         
  | process_flag = 'n' then    /* No data returned for this key      */         
  do                                                                            
   say '  NOEXTNT = no data'                                                    
  /*Add data length for this field name to the return work area index*/         
   index2_rwadata = index2_rwadata + CSILENF.fkc                                
   return                                                                       
  end                                                                           
                                                                                
 pos_of_noextnt = index2_rwadata /* Position of xhrba data           */         
 numvol = CSILENF.fkc           /* How many volsers are listed?      */         
 noextnt_list = ''              /* Clear noextnt list                */         
 Do vl = 1 to numvol            /*                                   */         
  /* Copy noextnt to a temp work area                                */         
  temp_field = substr(temp_worka,pos_of_noextnt,1)                              
  temp_field = C2X(temp_field)  /* Convert to hex                    */         
  /* When IGGCSI00 returns a field of x'F's the field is equal to 0  */         
  if pos('FF',temp_field) > 0 then noextnt_data= 'Candidate'                    
  else                                                                          
   do                                                                           
    noextnt_data= C2D(SUBSTR(temp_worka,pos_of_noextnt,1))                      
    noextnt.vl = noextnt_data   /* Add number of extents to array  */           
    noextnt.0  = vl             /* Count of noextnt entries        */           
   end                                                                          
                                                                                
  pos_of_noextnt = pos_of_noextnt + 1 /* pos of next noextnt entry */           
  if vollist.0 > 0 then                                                         
   noextnt_list=noextnt_list 'Vol' vollist.vl 'NOEXTNT = 'noextnt_data          
  else                                                                          
   noextnt_list=noextnt_list 'Vol' vl 'NOEXTNT = 'noextnt_data                  
 End                                                                            
                                                                                
 total_extents = 0                                                              
 do vl = 1 to noextnt.0        /* Sum number of extents for data set*/          
  total_extents = total_extents + noextnt.vl                                    
 end                                                                            
                                                                                
 say '  Data set has 'total_extents' extents on 'noextnt.0' volumes'            
 words_per_entry = 5           /* Number of words in list per entry  */         
 left_justify = 25             /* Left justify each entry            */         
 entries_per_line = 3          /* Number of entries per print line   */         
 first_line_prefix = ''                                                         
 other_line_prefix = ''                                                         
 list_2parse = noextnt_list                                                     
 Call Print_List                                                                
 index2_rwadata = pos_of_noextnt     /* Save index into rwa data     */         
return                                                                          
                                                                                
/*-------------------------------------------------------------------*/         
/* Process the EXTENT data                                           */         
/*-------------------------------------------------------------------*/         
Parse_EXTENT:                                                                   
/*                                                                              
 queue  temp_worka                                                              
 queue ''                                                                       
 "execio * diskw iggdata (finis"                                                
 */                                                                             
 numeric digits 12             /* Set processing to 12 digits        */         
 process_flag = 'y'                                                             
 vvr_extent_count = 0          /* Set variable to zero               */         
 volcount = 0                  /* Set variable to zero               */         
                                                                                
 If dtype <> 'DATA',           /* Is this ds valid for RBA?          */         
  & dtype <> 'INDEX',                                                           
  & dtype <> 'AIX' then process_flag = 'n' /* Turn off process flag  */         
 if CSILENF.fkc = 0,           /* No data returned for this key      */         
  | process_flag = 'n' then    /* No data returned for this key      */         
  do                                                                            
   say '  EXTENT = no data'                                                     
 /*Add data length for this field name to the return work area index */         
   index2_rwadata = index2_rwadata + CSILENF.fkc                                
   return                                                                       
  end                                                                           
                                                                                
 end_of_VVRdata = index2_rwadata + CSILENF.fkc                                  
 Do while index2_rwadata < end_of_VVRdata                                       
 volcount = volcount + 1                                                        
 /* The first two bytes of the extent data is the length of returned            
    data     */                                                                 
 extent_len= c2d(substr(temp_worka,index2_rwadata,2))                           
                                                                                
 index2_rwadata =  index2_rwadata + 2                                           
 If extent_len > 0 then                                                         
  do                                                                            
  /* Copy extent data to another variable as this helps when                    
     debugging the substr statements                                 */         
  extent_data = substr(temp_worka,index2_rwadata,extent_len)                    
  /* Calculate number of extents by dividing length of returned extent          
     data by 20.  CSI returns 20 bytes of data for each extent       */         
  Number_of_extents = extent_len/20                                             
  /* Count the number of extents reported from each VVR entry        */         
  vvr_extent_count = vvr_extent_count + Number_of_extents                       
  index2_extdata = 1           /* Set index into returned extent data*/         
                                                                                
 /* EXTENT data is a variable length field with the following format:           
    a 2 byte length (this length is the total length of the                     
    information that follows, and divided by 20, is the number of               
    extents in this VVR), followed by a 20 byte field for each extent.          
    The 20 byte field describes the start and the end of the extent,            
    in the form SSCCHHCCHHTTSSSSEEEE, where SS is the extent sequence           
    number ,CCHHCCHH is the low and high cylinder and head, TT is the           
    number of tracks, SSSS is the starting RBA and EEEE is the ending           
    RBA. Stephen Branch - SSD Software Requirements and Strategy     */         
                                                                                
  say '  '                                                                      
  if vollist.0 > 0 then                                                         
  say '  Data set has' Number_of_extents 'extents on 'vollist.volcount          
  else                                                                          
  say '  Data set has' Number_of_extents 'extents on volume 'volcount           
                                                                                
   Do while Number_of_extents > 0                                               
    Extent_Seq_numb  = C2D(SUBSTR(extent_data,(index2_extdata),2))              
                                                                                
    Extent_low_cchhr = C2X(SUBSTR(extent_data,(index2_extdata+2),4))            
    Extent_low_cchhr= right(Extent_low_cchhr,8,0) /* pad to 8 digits */         
    Extent_high_cchhr=D2X(C2D(SUBSTR(extent_data,(index2_extdata+6),4)))        
    Extent_high_cchhr= right(Extent_high_cchhr,8,0) /*pad to 8 digits*/         
    Extent_tracks    = C2D(SUBSTR(extent_data,(index2_extdata+10),2))           
    Extent_Start_RBA = C2D(SUBSTR(extent_data,(index2_extdata+12),4))           
    Extent_End_RBA   = SUBSTR(extent_data,(index2_extdata+16),4)                
                                                                                
    Extent_End_RBA   = c2d(Extent_End_RBA)                                      
    dr1 = '  Extent Seq number  =' left(Extent_Seq_numb,8)                      
    dr1 = dr1 '  Extent tracks      =' Extent_tracks                            
    dr2 = '  Extent low cchhr   =' Extent_low_cchhr                             
    dr2 = dr2 '  Extent Start RBA   =' Extent_Start_RBA                         
    dr3 ='  Extent high cchhr  =' Extent_high_cchhr                             
    dr3 = dr3 '  Extent End RBA     =' Extent_End_RBA                           
    say dr1                                                                     
    say dr2                                                                     
    say dr3                                                                     
    If Number_of_extents > 0 then                                               
     do                                                                         
      Number_of_extents = Number_of_extents -1 /* Decrement ext cntr */         
      index2_extdata = index2_extdata + 20     /* Next data entry    */         
      index2_rwadata = index2_rwadata + 20                                      
     end                                                                        
   end                                                                          
  end                                                                           
 else EXTENT = 'Unsure data'                                                    
 end                                                                            
                                                                                
 if vvr_extent_count > 0 then                                                   
  say '  Number of extents from all VVR entries is 'vvr_extent_count            
                                                                                
return                                                                          
                                                                                
/*------------------------------------------------------------------*/          
/* Process the XACIFLAG data                                        */          
/*------------------------------------------------------------------*/          
Parse_XACIFLAG:                                                                 
 process_flag = 'y'                                                             
 If dtype <> 'DATA',           /* Is this ds valid for RBA?          */         
  & dtype <> 'INDEX',                                                           
  & dtype <> 'AIX' then process_flag = 'n' /* Turn off process flag  */         
 if CSILENF.fkc = 0,           /* No data returned for this key      */         
  | process_flag = 'n' then    /* No data returned for this key      */         
  do                                                                            
   say '  XACIFLAG = no data'                                                   
 /*Add data length for this field name to the return work area index */         
   index2_rwadata = index2_rwadata + CSILENF.fkc                                
   return                                                                       
  end                                                                           
                                                                                
 srcflg= SUBSTR(temp_worka,index2_rwadata,CSILENF.fkc)                          
 XACIFLAG = bitand(srcflg,"40"x)                                                
 If XACIFLAG  <> "00"x then                                                     
  say '  XACIFLAG - Data set is EA (extended addressing)'                       
 else say '  XACIFLAG - Data set is not EA (extended addressing)'               
 /*Add data length for this field name to the return work area index */         
 index2_rwadata = index2_rwadata + CSILENF.fkc                                  
return                                                                          
                                                                                
/*------------------------------------------------------------------*/          
/* Process the COMPIND data                                         */          
/*------------------------------------------------------------------*/          
Parse_COMPIND:                                                                  
 process_flag = 'y'                                                             
 If dtype <> 'DATA',           /* Is this ds valid for RBA?          */         
  & dtype <> 'INDEX',                                                           
  & dtype <> 'AIX' then process_flag = 'n' /* Turn off process flag  */         
 if CSILENF.fkc = 0,           /* No data returned for this key      */         
  | process_flag = 'n' then    /* No data returned for this key      */         
  do                                                                            
   say '  COMPIND = no data'                                                    
 /*Add data length for this field name to the return work area index */         
   index2_rwadata = index2_rwadata + CSILENF.fkc                                
   return                                                                       
  end                                                                           
                                                                                
 srcflg= SUBSTR(temp_worka,index2_rwadata,CSILENF.fkc)                          
 COMPIND1 = bitand(srcflg,"40"x)                                                
 COMPIND2 = bitand(srcflg,"02"x)                                                
                                                                                
 If COMPIND1 = "40"x then say '  COMPIND - Data set is extended format'         
 If COMPIND2 = "20"x then say '  COMPIND - Data set is compressible'            
 If srcflg = "00"x then say '  COMPIND - No flags set'                          
                                                                                
 /*Add data length for this field name to the return work area index */         
 index2_rwadata = index2_rwadata + CSILENF.fkc                                  
                                                                                
return                                                                          
                                                                                
/*------------------------------------------------------------------*/          
/* Process the AMDCIREC data                                        */          
/*------------------------------------------------------------------*/          
Parse_AMDCIREC:                                                                 
 numeric digits 12             /* Set processing to 12 digits        */         
 process_flag = 'y'                                                             
 If dtype <> 'DATA',           /* Is this ds valid for RBA?          */         
  & dtype <> 'INDEX',                                                           
  & dtype <> 'AIX' then process_flag = 'n' /* Turn off process flag  */         
 if CSILENF.fkc = 0,           /* No data returned for this key      */         
  | process_flag = 'n' then    /* No data returned for this key      */         
  do                                                                            
   say '  AMDCIREC = no data'                                                   
 /*Add data length for this field name to the return work area index */         
   index2_rwadata = index2_rwadata + CSILENF.fkc                                
   return                                                                       
  end                                                                           
                                                                                
 AMDCIREC = SUBSTR(temp_worka,index2_rwadata,CSILENF.fkc)                       
 CISZ = C2D(SUBSTR(AMDCIREC,1,4))                                               
 MAXRECSZ = C2D(SUBSTR(AMDCIREC,5,4))                                           
 Say '  CI Size  =' CISZ                                                        
 Say '  Max RECSZ  =' MAXRECSZ                                                  
                                                                                
 /*Add data length for this field name to the return work area index */         
 index2_rwadata = index2_rwadata + CSILENF.fkc                                  
                                                                                
return                                                                          
                                                                                
/*------------------------------------------------------------------*/          
/* Process the AMDKEY data                                          */          
/*------------------------------------------------------------------*/          
Parse_AMDKEY:                                                                   
 numeric digits 12             /* Set processing to 12 digits        */         
 process_flag = 'y'                                                             
 If dtype <> 'DATA',           /* Is this ds valid for RBA?          */         
  & dtype <> 'INDEX',                                                           
  & dtype <> 'AIX' then process_flag = 'n' /* Turn off process flag  */         
 if CSILENF.fkc = 0,           /* No data returned for this key      */         
  | process_flag = 'n' then    /* No data returned for this key      */         
  do                                                                            
   say '  AMDKEY = no data'                                                     
 /*Add data length for this field name to the return work area index */         
   index2_rwadata = index2_rwadata + CSILENF.fkc                                
   return                                                                       
  end                                                                           
                                                                                
 AMDKEY = SUBSTR(temp_worka,index2_rwadata,CSILENF.fkc)                         
 RKP = C2D(SUBSTR(AMDKEY,1,2))                                                  
 KEYLEN = C2D(SUBSTR(AMDKEY,3,2))                                               
 Say '  Relative key position =' RKP                                            
 Say '  Key length =' KEYLEN                                                    
                                                                                
 /*Add data length for this field name to the return work area index */         
 index2_rwadata = index2_rwadata + CSILENF.fkc                                  
                                                                                
return                                                                          
/*------------------------------------------------------------------*/          
/* Non-VSAM attribute information (NVSMATTR) parsing routine        */          
/*------------------------------------------------------------------*/          
Parse_NVSMATTR:                                                                 
 numeric digits 12             /*  Set processing to 12 digits       */         
 If CSILENF.fkc = 0 then       /* No data returned for this key      */         
  do                                                                            
   say '  'CSIFLD1.fkc ' = no data'                                             
 /*Add data length for this field name to the return work area index */         
 /*index2_rwadata = index2_rwadata + CSILENF.fkc  */                            
   return                                                                       
  end                                                                           
                                                                                
 somefield = SUBSTR(temp_worka,index2_rwadata,CSILENF.fkc)                      
 hexfield = C2X(somefield)                                                      
 if datatype(somefield) = 'CHAR' then char_field = "c'"somefield"'"             
 else char_field = ''                                                           
 explain_field = ''                                                             
 if somefield = 'H' then explain_field = 'Active GDS'                           
 if somefield = 'N' then explain_field = 'Deferred GDS'                         
 if somefield = 'M' then explain_field = 'Rolled-off GDS'                       
 if somefield = 'L' then explain_field = 'Extended (PDSE)'                      
 if somefield = 'P' then explain_field = 'POSIX data set'                       
 if hexfileld = '00' then explain_field = 'Simple non-VSAM data set'            
 Say '  'CSIFLD1.fkc "= "char_field "x'"hexfield"'" explain_field               
                                                                                
 /*Add data length for this field name to the return work area index */         
 index2_rwadata = index2_rwadata + CSILENF.fkc                                  
                                                                                
return                                                                          
                                                                                
/*------------------------------------------------------------------*/          
/* ASSOCSYM parsing routine                                                     
   A repeating list of catalog records associated with this entry.              
   Consists of a 1-byte value similar to field name ENTYPE, followed            
   by the 44-byte name of the association. If the name contains system          
   symbolics, they are not resolved.  Each entry is 45 bytes long.  */          
/*------------------------------------------------------------------*/          
Parse_ASSOCSYM:                                                                 
 numeric digits 12             /*  Set processing to 12 digits       */         
 If CSILENF.fkc = 0 then       /* No data returned for this key      */         
  do                                                                            
   say '  'CSIFLD1.fkc ' = no data'                                             
 /*Add data length for this field name to the return work area index */         
 /*index2_rwadata = index2_rwadata + CSILENF.fkc  */                            
   return                                                                       
  end                                                                           
                                                                                
 somefield = SUBSTR(temp_worka,index2_rwadata,CSILENF.fkc)                      
 assocsym_len = length(somefield) % 45 /* Total # of assoc entries   */         
 do asym = 1 to assocsym_len                                                    
  asym_eoffset = ((asym*45)+1)-45    /* Alias entry type start loc   */         
  asym_offset = asym_eoffset + 1     /* Start of catalog association */         
  /*  The next 4 statements are for testing purposes                            
  dr ='asym =' asym                                                             
  dr = dr 'asym_eoffset  =' asym_eoffset                                        
  dr = dr 'asym_offset   =' asym_offset                                         
  if asym < 5 then say dr                                                       
  */                                                                            
  asym_field  = substr(somefield,asym_eoffset,1)                                
  asym_assoc  = substr(somefield,asym_offset,44)                                
  asym_xfield = C2X(asym_field)                                                 
  dr ="   Catalog ASSOCSYM."asym "= "asym_field "(x'"asym_xfield"')"            
  dr = dr asym_assoc                                                            
  say dr                                                                        
 end                                                                            
                                                                                
 /*Add data length for this field name to the return work area index */         
 index2_rwadata = index2_rwadata + CSILENF.fkc                                  
                                                                                
return                                                                          
                                                                                
/*------------------------------------------------------------------*/          
/* Generic parsing routine                                          */          
/*------------------------------------------------------------------*/          
Parse_Generic:                                                                  
 numeric digits 12             /*  Set processing to 12 digits       */         
 If CSILENF.fkc = 0 then       /* No data returned for this key      */         
  do                                                                            
   say '  'CSIFLD1.fkc ' = no data'                                             
 /*Add data length for this field name to the return work area index */         
 /*index2_rwadata = index2_rwadata + CSILENF.fkc  */                            
   return                                                                       
  end                                                                           
                                                                                
 somefield = SUBSTR(temp_worka,index2_rwadata,CSILENF.fkc)                      
 hexfield = C2X(somefield)                                                      
 Say '  'CSIFLD1.fkc '= x'''hexfield''''                                        
                                                                                
 /*Add data length for this field name to the return work area index */         
 index2_rwadata = index2_rwadata + CSILENF.fkc                                  
                                                                                
return                                                                          
                                                                                
/*------------------------------------------------------------------*/          
/* Parse CAXWA flags                                                */          
/*------------------------------------------------------------------*/          
Parse_CatFlags:                                                                 
 numeric digits 12             /*  Set processing to 12 digits       */         
 If CSILENF.fkc = 0 then       /* No data returned for this key      */         
  do                                                                            
   say '  'CSIFLD1.fkc ' = no data'                                             
 /*Add data length for this field name to the return work area index */         
 /*index2_rwadata = index2_rwadata + CSILENF.fkc  */                            
   return                                                                       
  end                                                                           
                                                                                
 somefield = SUBSTR(temp_worka,index2_rwadata,CSILENF.fkc)                      
 hexfield = C2X(somefield)                                                      
 Say '  'CSIFLD1.fkc '= x'''hexfield''''                                        
 catflagx = x2b(hexfield)                         /*Convert to binary*/         
 Say '  'Bit String  '= b'''catflagx''''                                        
 Say '                 First Byte explains catalog is ....'                     
 Say '                 1xxxxxxx   open'                                         
 Say '                 x1xxxxxx   the master catalog on this system'            
 Say '                 xx1xxxxx   active in In-Storage Cache'                   
 Say '                 xxx1xxxx   active in VLF'                                
 Say '                 xxxx1xxx   ECS-active'                                   
 Say '                 xxxxx1xx   open in RLS mode'                             
 Say '                 xxxxxx1x   deleted'                                      
 Say '                 xxxxxxx1   locked'                                       
 Say '                            Second Byte Reserved'                         
                                                                                
 /*Add data length for this field name to the return work area index */         
 index2_rwadata = index2_rwadata + CSILENF.fkc                                  
                                                                                
return                                                                          
                                                                                
/*------------------------------------------------------------------*/          
/* Parse tape volume catalogs in DFSMS/MVS field names              */          
/*------------------------------------------------------------------*/          
Parse_Library:                                                                  
 numeric digits 12             /*  Set processing to 12 digits       */         
 If CSILENF.fkc = 0 then       /* No data returned for this key      */         
  do                                                                            
   say '  'CSIFLD1.fkc ' = no data'                                             
 /*Add data length for this field name to the return work area index */         
 /*index2_rwadata = index2_rwadata + CSILENF.fkc  */                            
   return                                                                       
  end                                                                           
                                                                                
 somefield = SUBSTR(temp_worka,index2_rwadata,CSILENF.fkc)                      
 hexfield = C2X(somefield)                                                      
 Say '  'CSIFLD1.fkc '= x'''hexfield'''  ' somefield                            
                                                                                
 /*Add data length for this field name to the return work area index */         
 index2_rwadata = index2_rwadata + CSILENF.fkc                                  
                                                                                
return                                                                          
                                                                                
/*-------------------------------------------------------------------*/         
/* Parse and print a list of data returned by the catalog search     */         
/*-------------------------------------------------------------------*/         
Print_List:                                                                     
 word_count = 0                /* Counter for words in list to parse */         
 print_line = ''               /* print line variable                */         
 lines_printed = 0             /* print line counter                 */         
 entry_count = 0               /* Count entries per print line       */         
 do while word_count < words(list_2parse) /* Loop thru list to print */         
  print_entry = ''             /* Clear print entry variable         */         
  do sub_entry = 1 to words_per_entry /* Build print entry           */         
   print_entry = print_entry word(list_2parse,(word_count+1))                   
   word_count = word_count + 1                                                  
  end                                                                           
  /* If required left justify  print entry if so required            */         
  if left_justify > 0 then print_entry = left(print_entry,left_justify)         
  print_line = print_line print_entry /* Add entry to print line     */         
  entry_count = entry_count + 1 /* Count number of entries on line   */         
  if entry_count = entries_per_line then /* Reached entries per line?*/         
   do                          /* Print the formatted output line    */         
    if lines_printed = 0 then  /* First line to be printed?          */         
     do                                                                         
      say '  '                                                                  
      say first_line_prefix||print_line                                         
     end                                                                        
    else say other_line_prefix||print_line                                      
    lines_printed = lines_printed + 1  /* Count lines printed        */         
    entry_count = 0            /* Clear entries per print line       */         
    print_line = ''            /* Clear the print line               */         
   end                                                                          
 end                                                                            
                                                                                
 if entry_count > 0 then       /* Are there entries still to print   */         
  do                                                                            
   if lines_printed = 0 then   /* First line to be printed?          */         
    do                                                                          
     say '  '                                                                   
     say first_line_prefix||print_line                                          
    end                                                                         
   else say other_line_prefix||print_line                                       
  end                                                                           
 return                                                                         
                                                                                
/*------------------------------------------------------------------*/          
/* Write Help Text for this exec                                    */          
/*------------------------------------------------------------------*/          
Write_Help:                                                                     
say '                                                                  '        
say ' Description: This rexx exec is used to call the catalog search'           
say '              interface.                                          '        
say ' Input: filter key and catalog field names (optional)             '        
say '                                                                  '        
say '        The order of fieldnames is the way they are returned from '        
say '        IGGCSI00 and thus the way they are parsed by the CATSRCHI '        
say '        exec.  It is recommended that the VOLSER keyword be first '        
say '        if it is included as the volser info is used several      '        
say '        times in reporting in the exec. For example:              '        
say '       %CATSRCNW KLMTEST1.KSDS02 FIELDNAMES(XHARBA XHURBA NOEXTNT)'        
say '                                                                  '        
say ' Output: Data returned from the catalog search interface for      '        
say '         which a parsing routine exists.                          '        
say '                                                                  '        
say ' Variables:                                                       '        
say ' Fieldnames() As documented in the manual Managing Catalogs, i.e. '        
say '       %CATSRCNW KLMTEST1.KSDS02 FIELDNAMES(XHARBA XHURBA NOEXTNT)'        
say '                                                                  '        
say ' Output: Data returned from the catalog search interface for      '        
say '         which a parsing routine exists.                          '        
say '                                                                  '        
Write_Return_Codes:                                                             
say ' Condition Codes:                                                 '        
say ' 20 - A filter key was not passed to the exec.                    '        
say ' 21 - The catalog search interface did not find any matches based '        
say '      on the filter key                                           '        
say ' 22 - The field passed in the INIT WORK() parm is not numeric.    '        
say ' xx - Any non-zero return code set by the catalog search interface'        
say '                                                                  '        
if exitrc > 0 then                                                              
 do                                                                             
  say 'Ending with a return code of 'exitrc                                     
  say '  '                                                                      
  exit(exitrc)                                                                  
 end                                                                            
                                                                                
exit                                                                            
